function [RND, K2] = ExtractRND(K,CallPrice,type)

% Function to extract the Risk Neutral Density from Call prices.
% Inputs
%   K = vector of strikes (size N)
%   CallPrice = vector of call prices (size N).
%   type = Interpolation type ('linear', 'spline', 'cubic').
% Outputs
%   K2 = vector of strikes (size N-2).
%   RND = vector of RND (size N-2).


% Calculate the first derivatives of calls w.r.t. strike K. 
% Use central differences
for i=2:length(K)-1
	dK = K(i+1) - K(i-1);
	dC = CallPrice(i+1) - CallPrice(i-1);
	dCdK(i-1) = dC/dK;
end

% Calculate the risk neutral density by central finite differences.
for i=2:length(dCdK)-1;
	dK  = K(i+1) - K(i-1);
	dC2 = dCdK(i+1) - dCdK(i-1);
	RND(i-1) = dC2/dK;
end
clear dCdK
K = K(3:end-2);

% Remove negative values for the RND.
% Interpolate to fill them in.
All = 1:length(RND);            % All indices for RND
Neg = find(RND<0);              % Indices with RND <  0
Pos = setdiff(All,Neg);         % Indices with RND => 0
Kneg = K(Neg);                  % Strikes with RND < 0
Kpos = K(Pos);                  % Strikes with RND => 0
RNDneg = RND(Neg);              % RND < 0
RNDpos = RND(Pos);              % RND => 0

% Replace negative values of RND with interpolated values.
RNDi = interp1(Kpos,RNDpos,Kneg,type);
RNDnew = sortrows([Kpos' RNDpos'; Kneg' RNDi'],1);

% The outputs
K2 = RNDnew(:,1);
RND = RNDnew(:,2);
