% Risk neutral extraction from implied volatilities.
% Steps
%   1. Fit either a deterministic volatility function (DVF) or Jim Gatheral's
%      stochastic inspired volatility (SVI) to the implied volatilities.
%   2. Input Strikes, Implied volatilities and option features.
%   3. Fit either a DVF or SVI to the implied volatilities.
%   4. Convert the implied vols to market prices using Black Scholes.
%   5. Extrapolate implied vols beyond the strike range using flat extrapolation
%   6. Extract the Risk Neutral Density from the extrapolated implied vols and
%      verify that it integrates to one.
%   7. Check that the RND recovers the call prices

clc; clear;

% Inline function for the Black Scholes call
BSCall = inline('s*exp(-q*T)*normcdf((log(s/K) + (r-q+v^2/2)*T)/v/sqrt(T)) - K*exp(-r*T)*normcdf((log(s/K) + (r-q+v^2/2)*T)/v/sqrt(T) - v*sqrt(T))',...
	     's','K','r','q','v','T');

% Step 1. 
% Specify either
% 'DVF' = quadratic function (deterministic volatility function).
% 'SVI' = Stochastic volatility inspired (Jim Gatheral).
VolType = 'SVI'

% Step 2.
% Imput Strikes and Implied vols for Google, and option features
K = [300 310 320 330 340 350 360 370 380 390 400 400 410 410 ...
	 420 420 430 430 440 440 450 450 460 470 480 490 500 510];

IV = [0.6842 0.6566 0.6197 0.5928 0.5311 0.5205 0.4838 0.4669 0.4420 0.4267 0.4027...
	  0.4120 0.3974 0.4019 0.3894 0.3918 0.3783 0.3815 0.3714 0.3802 0.3643 0.3663...
	  0.3653 0.3730 0.3705 0.3857 0.3783 0.4006];

% Option and stock price parameters Parameters
T  = 0.04;       % Maturity
rf = 0.01;       % Risk free rate
S  = 423.19;     % Spot price

% Step 3.
% Fit the DVF or the SVI to the implied vols
if strcmp(VolType,'DVF')
	beta = regstats(IV, K, 'purequadratic');
	yhat = beta.yhat;
	B = beta.beta;
elseif strcmp(VolType,'SVI')
	start = 0.5.*[1 1 1 1 1];
	options = optimset('MaxFunEvals', 1e5, 'MaxIter', 1e5);
	[params feva] = fminsearch(@(b) findSVIparams(b,K,IV,S,rf,T), start,options);
	a = params(1);
	b = params(2);
	r = params(3);
	m = params(4);
	s = params(5);
end

% Step 4.
% Convert to Black Scholes call prices
% Range of strikes to fit.
dK = 0.1;
K2 = [min(K):dK:max(K)];

% Fitted volatilities and Black Scholes prices.
for i=1:length(K2)
	if strcmp(VolType,'DVF')
		Vol(i) = B(1) + B(2)*K2(i) + B(3)*K2(i)^2;
	elseif strcmp(VolType,'SVI')
		Vol(i) = SVI(K2(i),S,rf,T,a,b,r,m,s);
	end
	BSC(i) = BSCall(S,K2(i),rf,0,Vol(i),T);
end

% Step 5.
% Extrapolate the wings of the IV using lognormal tails (flat lines)
% The strikes go from 300 to 510, extrapolate to 200 and 600.
LoK = [200:K2(1)-1];
HiK = [K2(end)+1:600];
K2 = [LoK K2 HiK];

% Extrapolate volatilites on the lower and upper wings.
LoVol = Vol(1)  .*ones(1,length(LoK));
HiVol = Vol(end).*ones(1,length(HiK));
Vol = [LoVol Vol HiVol];

% Extrapolate the calls on the lower and upperwings.
for i=1:length(LoK);
%	LoBSC(i) = BSCall(Vol(1), S, LoK(i), rf, 0, T, 'C');
	LoBSC(i) = BSCall(S,LoK(i),rf,0,Vol(1),T);
end
for i=1:length(HiK);
%	HiBSC(i) = BSCall(Vol(end), S, HiK(i), rf, 0, T, 'C');
	HiBSC(i) = BSCall(S,HiK(i),rf,0,Vol(end),T);
end
BSCext = [LoBSC BSC HiBSC];
clear LoK HiK LoVol HiVol LoBSC HiBSC

% Plot the Implied volatilities.
plot(K, IV, 'x', K2, Vol, 'r-')
legend('Market Implied Vol', 'Fitted IV') 
xlabel('Strike Price')
ylabel('Implied Volatility')
clear beta yhat B
pause

% Plot the call prices using the fitted implied volatility.
plot(K2, BSCext)
if strcmp(VolType, 'DVF')
	legend('Market Call Price: B-S using Implied Vol fitted with DVF')
elseif strcmp(VolType, 'SVI')
	legend('Market Call Price: B-S using Implied Vol fitted with SVI')
end	
xlabel('Strike Price')
ylabel('Call Price')
pause

% Step 6.
% Extract the RND.
[RND, K2] = ExtractRND(K2,BSCext,'cubic');

% Plot the RND.  The lognormal tails are a little thin.
plot(K2, RND)
legend('Risk Neutral Density');
xlabel('Strike Price')
ylabel('RND')
pause

% Verify the integral is 1.
AreaUnderRND = trapz(K2(1:length(RND)), RND)
 
% Verify that the RND recovers the call prices.
for i=1:length(K);
    Kstar = K(i);
    I = find(K2<=Kstar);
	NewStrike = K2;
	NewStrike(I) = 0;
    integrand = max(NewStrike - Kstar,0).*RND;
    RecoveredCall(i) = exp(-rf*T)*trapz(integrand).*dK;
	MarketCall(i) = BSCall(S,K(i),rf,0,IV(i),T);
	PercentageError(i) = (RecoveredCall(i) - MarketCall(i))/MarketCall(i)*100;
end;
CallChecks = [MarketCall' RecoveredCall' PercentageError']

% Scattergram showing the market calls and recovered calls are very close.
scatter(MarketCall, RecoveredCall);
xlabel('Call price from the market');
ylabel('Call price recovered from the RND')

% Regression R2 to show how close market and recovered calls are.
RegCalls = regstats(RecoveredCall, MarketCall, 'linear');
R2 = RegCalls.rsquare

